@@ -28,15 +28,37 @@
 #include "thermald.h"
 #include "thd_preference.h"
 #include "thd_engine.h"
+#include "thd_engine_adaptive.h"
 #include "thd_engine_default.h"
 #include "thd_parse.h"
+#include <sys/file.h>
+
+#define EXIT_UNSUPPORTED 2
+
+// getdtablesize() is removed from bionic/libc in LPDK*/
+// use POSIX alternative available. Otherwise fail
+#  ifdef _POSIX_OPEN_MAX
+#   define   getdtablesize()	(_POSIX_OPEN_MAX)
+# endif
+// for AID_* constatns
+#include <cutils/android_filesystem_config.h>
+
+// getdtablesize() is removed from bionic/libc in LPDK*/
+// use POSIX alternative available. Otherwise fail
+#  ifdef _POSIX_OPEN_MAX
+#   define   getdtablesize()	(_POSIX_OPEN_MAX)
+# endif
 
 // poll mode
 int thd_poll_interval = 4; //in seconds
-static int pid_file_handle;
 
-// Thermal engine
-cthd_engine *thd_engine;
+bool thd_ignore_default_control = false;
+bool workaround_enabled = false;
+bool disable_active_power = false;
+bool ignore_critical = false;
+bool power_floor_enable = false;
+
+static int pid_file_handle;
 
 // Stop daemon
 static void daemonShutdown() {
@@ -51,11 +73,11 @@ static void daemonShutdown() {
 static void signal_handler(int sig) {
 	switch (sig) {
 	case SIGHUP:
-		thd_log_warn("Received SIGHUP signal. \n");
+		thd_log_warn("Received SIGHUP signal.\n");
 		break;
 	case SIGINT:
 	case SIGTERM:
-		thd_log_info("Daemon exiting \n");
+		thd_log_info("Daemon exiting\n");
 		daemonShutdown();
 		exit(EXIT_SUCCESS);
 		break;
@@ -140,8 +162,9 @@ static void print_usage(FILE* stream, int exit_code) {
 	fprintf(stream, "  --help Display this usage information.\n"
 			"  --version Show version.\n"
 			"  --no-daemon No daemon.\n"
-			"  --poll-interval poll interval 0 to disable.\n"
-			"  --exclusive_control to act as exclusive thermal controller. \n");
+			"  --poll-interval Poll interval 0 to disable.\n"
+			"  --exclusive_control To act as exclusive thermal controller.\n"
+			"  --config-file Configuration file to use other than the default config.\n");
 
 	exit(exit_code);
 }
@@ -152,8 +175,13 @@ int main(int argc, char *argv[]) {
 	bool no_daemon = false;
 	bool exclusive_control = false;
 	bool test_mode = false;
+	bool is_privileged_user = false;
+	char *conf_file = NULL;
+	bool ignore_cpuid_check = false;
+	bool adaptive = false;
+	int ret;
 
-	const char* const short_options = "hvnp:de";
+	const char* const short_options = "hvnp:detc:ia";
 	static struct option long_options[] = {
 			{ "help", no_argument, 0, 'h' },
 			{ "version", no_argument, 0, 'v' },
@@ -161,12 +189,15 @@ int main(int argc, char *argv[]) {
 			{ "poll-interval", required_argument, 0, 'p' },
 			{ "exclusive_control", no_argument, 0, 'e' },
 			{ "test-mode", no_argument, 0, 't' },
+			{ "config-file", required_argument, 0, 'c' },
+			{ "ignore-cpuid-check", no_argument, 0, 'i'},
+			{ "ignore-default-control", no_argument, 0, 'd'},
+			{ "adaptive", no_argument, 0, 'a'},
 			{ NULL, 0, NULL, 0 } };
 
 	if (argc > 1) {
 		while ((c = getopt_long(argc, argv, short_options, long_options,
 				&option_index)) != -1) {
-			int this_option_optind = optind ? optind : 1;
 			switch (c) {
 			case 'h':
 				print_usage(stdout, 0);
@@ -187,6 +218,18 @@ int main(int argc, char *argv[]) {
 			case 't':
 				test_mode = true;
 				break;
+			case 'c':
+				conf_file = optarg;
+				break;
+			case 'i':
+				ignore_cpuid_check = true;
+				break;
+			case 'd':
+				thd_ignore_default_control = true;
+				break;
+			case 'a':
+				adaptive = true;
+				break;
 			case -1:
 			case 0:
 				break;
@@ -195,12 +238,14 @@ int main(int argc, char *argv[]) {
 			}
 		}
 	}
-	if (getuid() != 0 && !test_mode) {
-		fprintf(stderr, "You must be root to run thermal daemon!\n");
-		exit(EXIT_FAILURE);
+
+	is_privileged_user = (getuid() == 0) || (getuid() == AID_SYSTEM);
+	if (!is_privileged_user && !test_mode) {
+		thd_log_error("You do not have correct permissions to run thermal daemon!\n");
+		exit(1);
 	}
 
-	if ((c = mkdir(TDRUNDIR, 0755)) != 0) {
+	if (mkdir(TDRUNDIR, 0755) != 0) {
 		if (errno != EEXIST) {
 			fprintf(stderr, "Cannot create '%s': %s\n", TDRUNDIR,
 					strerror(errno));
@@ -209,25 +254,34 @@ int main(int argc, char *argv[]) {
 	}
 	mkdir(TDCONFDIR, 0755); // Don't care return value as directory
 	if (!no_daemon) {
-		daemonize((char *) "/tmp/", (char *) "/tmp/thermald.pid");
+		daemonize((char *) "/data/vendor/thermal-daemon",
+				(char *) "/data/vendor/thermal-daemon/thermald.pid");
 	} else
 		signal(SIGINT, signal_handler);
 
 	thd_log_info(
 			"Linux Thermal Daemon is starting mode %d : poll_interval %d :ex_control %d\n",
 			no_daemon, thd_poll_interval, exclusive_control);
-
-	thd_engine = new cthd_engine_default();
-	if (exclusive_control)
-		thd_engine->set_control_mode(EXCLUSIVE);
-
-	thd_engine->set_poll_interval(thd_poll_interval);
-
-	// Initialize thermald objects
-	if (thd_engine->thd_engine_start(false) != THD_SUCCESS) {
-		thd_log_error("thermald engine start failed:\n");
-		exit(EXIT_FAILURE);
+	if (adaptive) {
+		ignore_cpuid_check= true;
+		ret = thd_engine_create_adaptive_engine((bool) ignore_cpuid_check, test_mode);
+		if (ret != THD_SUCCESS) {
+			thd_log_info("--adaptive option failed on this platform\n");
+			thd_log_info("Ignoring --adaptive option\n");
+			ret = thd_engine_create_default_engine((bool) ignore_cpuid_check,
+						       (bool) exclusive_control, conf_file);
+		}
+	} else {
+		ret = thd_engine_create_default_engine((bool) ignore_cpuid_check,
+					       (bool) exclusive_control, conf_file);
 	}
+	if (ret != THD_SUCCESS) {
+		if (ret == THD_ERROR)
+			exit(EXIT_UNSUPPORTED);
+		else
+			exit(EXIT_FAILURE);
+	}
+
 #ifdef VALGRIND_TEST
 	// lots of STL lib function don't free memory
 	// when called with exit().
@@ -242,7 +296,7 @@ int main(int argc, char *argv[]) {
 	for (;;)
 		sleep(0xffff);
 
-	thd_log_info("Linux Thermal Daemon is exiting \n");
+	thd_log_info("Linux Thermal Daemon is exiting\n");
 #endif
 	return 0;
 }
