@@ -42,6 +42,7 @@
 #define CDEV_DEF_BIT_UNIT_VAL	0x0080
 #define CDEV_DEF_BIT_DEBOUNCE_VAL	0x0100
 #define CDEV_DEF_BIT_PID_PARAMS	0x0200
+#define CDEV_DEF_BIT_WRITE_PREFIX	0x0400
 
 #define SENSOR_DEF_BIT_PATH		0x0001
 #define SENSOR_DEF_BIT_ASYNC_CAPABLE		0x0002
@@ -52,17 +53,34 @@ typedef struct {
 	double Kd;
 } pid_control_t;
 
+typedef struct {
+	std::string name;
+	double multiplier;
+	double offset;
+} thermal_sensor_link_t;
+
 typedef struct {
 	unsigned int mask;
 	std::string name;
 	std::string path;
 	bool async_capable;
+	bool virtual_sensor;
+	thermal_sensor_link_t sensor_link;
 } thermal_sensor_t;
 
+typedef struct {
+	int dependency;
+	std::string cdev;
+	std::string state;
+}trip_cdev_depend_t;
+
 typedef struct {
 	std::string type;
 	int influence;
 	int sampling_period;
+	int target_state_valid;
+	int target_state;
+	pid_param_t pid_param;
 } trip_cdev_t;
 
 typedef struct {
@@ -72,6 +90,7 @@ typedef struct {
 	trip_control_type_t control_type;
 	int influence;
 	std::string sensor_type;
+	trip_cdev_depend_t dependency;
 	std::vector<trip_cdev_t> cdev_trips;
 } trip_point_t;
 
@@ -99,14 +118,34 @@ typedef struct {
 	int debounce_interval;
 	bool pid_enable;
 	pid_control_t pid;
-
+	std::string write_prefix;
 } cooling_dev_t;
 
+typedef struct {
+	std::string name;
+	int valid;
+	int power_limit_min;
+	int power_limit_max;
+	int time_wind_min;
+	int time_wind_max;
+	int step_size;
+
+	int limit_1_valid;
+	int power_limit_1_min;
+	int power_limit_1_max;
+	int time_wind_1_min;
+	int time_wind_1_max;
+	int step_1_size;
+} ppcc_t;
+
 typedef struct {
 	std::string name;
 	std::string uuid;
 	std::string product_name;
-	int default_prefernce;
+	std::string product_sku;
+	int default_preference;
+	int polling_interval;
+	ppcc_t ppcc;
 	std::vector<thermal_sensor_t> sensors;
 	std::vector<thermal_zone_t> zones;
 	std::vector<cooling_dev_t> cooling_devs;
@@ -116,13 +155,16 @@ class cthd_parse {
 private:
 	std::string filename;
 	std::string filename_auto;
+	std::string filename_auto_conf;
 	std::vector<thermal_info_t> thermal_info_list;
 	int matched_thermal_info_index;
 	xmlDoc *doc;
 	xmlNode *root_element;
+	int auto_config;
 
 	int parse(xmlNode * a_node, xmlDoc *doc);
 	int parse_pid_values(xmlNode * a_node, xmlDoc *doc, pid_control_t *pid_ptr);
+	int parse_dependency_values(xmlNode * a_node, xmlDoc *doc, trip_cdev_depend_t *dependency);
 	int parse_new_trip_cdev(xmlNode * a_node, xmlDoc *doc,
 			trip_cdev_t *trip_cdev);
 
@@ -139,6 +181,8 @@ private:
 			thermal_info_t *info_ptr);
 	int parse_new_sensor(xmlNode * a_node, xmlDoc *doc,
 			thermal_sensor_t *info_ptr);
+	int parse_new_sensor_link(xmlNode * a_node, xmlDoc *doc,
+			thermal_sensor_link_t *info_ptr);
 	int parse_thermal_sensors(xmlNode * a_node, xmlDoc *doc,
 			thermal_info_t *info_ptr);
 	int parse_cooling_devs(xmlNode * a_node, xmlDoc *doc,
@@ -147,16 +191,21 @@ private:
 			thermal_zone_t *info_ptr);
 	int parse_new_platform(xmlNode * a_node, xmlDoc *doc, thermal_info_t *info);
 
+	int parse_ppcc(xmlNode * a_node, xmlDoc *doc, ppcc_t *ppcc);
+
 	void string_trim(std::string &str);
 	char *char_trim(char *trim);
+	bool match_product_sku(int index);
 
 public:
 	cthd_parse();
-	int parser_init();
+	int parser_init(std::string config_file);
 	void parser_deinit();
 	int start_parse();
 	void dump_thermal_conf();
 	bool platform_matched();
+	int get_polling_interval();
+	ppcc_t *get_ppcc_param(std::string name);
 	int zone_count() {
 		return thermal_info_list[matched_thermal_info_index].zones.size();
 	}
@@ -168,6 +217,14 @@ public:
 		return thermal_info_list[matched_thermal_info_index].sensors.size();
 	}
 
+	int thermal_conf_auto() {
+		return auto_config;
+	}
+
+	int thermal_matched_platform_index() {
+		return matched_thermal_info_index;
+	}
+
 	int set_default_preference();
 	int trip_count(unsigned int zone_index);
 	bool pid_status(int cdev_index);
