@@ -28,7 +28,8 @@
 cthd_sensor::cthd_sensor(int _index, std::string control_path,
 		std::string _type_str, int _type) :
 		index(_index), type(_type), sensor_sysfs(control_path.c_str()), sensor_active(
-				false), type_str(_type_str), async_capable(false), thresholds(0) {
+				false), type_str(std::move(_type_str)), async_capable(false), virtual_sensor(
+				false), thresholds(0), scale(1) {
 
 }
 
@@ -43,7 +44,7 @@ int cthd_sensor::sensor_update() {
 		if (sensor_sysfs.exists("temp")) {
 			return THD_SUCCESS;
 		} else {
-			thd_log_warn("sensor id %d: No temp sysfs for reading temp\n",
+			thd_log_msg("sensor id %d: No temp sysfs for reading temp\n",
 					index);
 			return THD_ERROR;
 		}
@@ -52,8 +53,8 @@ int cthd_sensor::sensor_update() {
 		if (sensor_sysfs.exists("")) {
 			return THD_SUCCESS;
 		} else {
-			thd_log_warn("sensor id %d: No temp sysfs for reading raw temp\n",
-					index);
+			thd_log_msg("sensor id %d %s: No temp sysfs for reading raw temp\n",
+					index, sensor_sysfs.get_base_path());
 			return THD_ERROR;
 		}
 	}
@@ -73,8 +74,8 @@ unsigned int cthd_sensor::read_temperature() {
 	std::istringstream(buffer) >> temp;
 	if (temp < 0)
 		temp = 0;
-	thd_log_debug("Sensor %s :temp %u \n", type_str.c_str(), temp);
-	return (unsigned int)temp;
+	thd_log_debug("Sensor %s :temp %u\n", type_str.c_str(), temp);
+	return (unsigned int)temp / scale;
 }
 
 void cthd_sensor::enable_uevent() {
@@ -95,6 +96,9 @@ int cthd_sensor::set_threshold(int index, int temp) {
 	std::stringstream thres;
 	int status = 0;
 
+	if (thd_engine->get_poll_interval())
+		return THD_SUCCESS;
+
 	if (!async_capable) {
 		return THD_ERROR;
 	}
@@ -119,3 +123,10 @@ void cthd_sensor::sensor_poll_trip(bool status) {
 	else
 		thd_engine->thd_engine_poll_disable(index);
 }
+
+void cthd_sensor::sensor_fast_poll(bool status) {
+	if (status)
+		thd_engine->thd_engine_fast_poll_enable(index);
+	else
+		thd_engine->thd_engine_fast_poll_disable(index);
+}
