@@ -1,5 +1,5 @@
 /*
- * thd_zone.cpp: thermal zone class implentation
+ * thd_zone.cpp: thermal zone class implementation
  *
  * Copyright (C) 2012 Intel Corporation. All rights reserved.
  *
@@ -37,60 +37,38 @@
 cthd_zone::cthd_zone(int _index, std::string control_path, sensor_relate_t rel) :
 		index(_index), zone_sysfs(control_path.c_str()), zone_temp(0), zone_active(
 				false), zone_cdev_binded_status(false), type_str(), sensor_rel(
-				rel), thd_model("") {
-	thd_log_debug("Added zone index:%d \n", index);
+				rel) {
+	thd_log_debug("Added zone index:%d\n", index);
 }
 
 cthd_zone::~cthd_zone() {
+	for (unsigned int i = 0; i < trip_points.size(); ++i) {
+		trip_points[i].delete_cdevs();
+	}
 	trip_points.clear();
 	sensors.clear();
 }
 
 void cthd_zone::thermal_zone_temp_change(int id, unsigned int temp, int pref) {
 	int i, count;
-	bool updated_max = false;
 	bool reset = false;
 
 	count = trip_points.size();
-	for (i = count - 1; i >= 0; --i) {
+	for (i = 0; i < count; ++i) {
 		cthd_trip_point &trip_point = trip_points[i];
-		if (trip_point.get_trip_type() == MAX) {
-			thd_model.add_sample(zone_temp);
-			if (thd_model.is_set_point_reached()) {
-				int set_point;
-				set_point = thd_model.get_set_point();
-				thd_log_debug("new set point %d \n", set_point);
-				trip_point.thd_trip_update_set_point(set_point);
-				updated_max = true;
-			}
-		}
 		trip_point.thd_trip_point_check(id, temp, pref, &reset);
 		// Force all cooling devices to min state
 		if (reset) {
-			zone_reset();
+			zone_reset(0);
 			break;
 		}
 	}
-	// Re-adjust polling thresholds
-	if (updated_max) {
-		for (i = count - 1; i >= 0; --i) {
-			cthd_trip_point &trip_point = trip_points[i];
-			if (trip_point.get_trip_type() == POLLING) {
-				thd_log_debug("new poll point %d \n",
-						thd_model.get_hot_zone_trigger_point());
-				trip_point.thd_trip_update_set_point(
-						thd_model.get_hot_zone_trigger_point());
-				trip_point.thd_trip_point_check(id, temp, pref, &reset);
-			}
-		}
-	}
 }
 
 void cthd_zone::update_zone_preference() {
 	if (!zone_active)
 		return;
 	thd_log_debug("update_zone_preference\n");
-	thd_model.update_user_set_max_temp();
 
 	for (unsigned int i = 0; i < sensors.size(); ++i) {
 		cthd_sensor *sensor;
@@ -127,14 +105,53 @@ int cthd_zone::read_user_set_psv_temp() {
 	return temp;
 }
 
+void cthd_zone::sort_and_update_poll_trip() {
+	thd_log_debug("sort_and_update_poll_trip: trip_points_size =%zu\n",
+			trip_points.size());
+
+	for (unsigned int i = 0; i < trip_points.size(); ++i) {
+		if (trip_points[i].get_trip_type() == POLLING) {
+			thd_log_debug("polling trip already present\n");
+			trip_points.erase(trip_points.begin() + i);
+			break;
+		}
+	}
+
+	if (trip_points.size()) {
+		unsigned int polling_trip = 0;
+
+		std::sort(trip_points.begin(), trip_points.end(), trip_sort);
+		if (trip_points.size())
+			polling_trip = trip_points[0].get_trip_temp();
+
+		unsigned int poll_offset = polling_trip * def_async_trip_offset_pct
+				/ 100;
+
+		if (poll_offset < def_async_trip_offset)
+			poll_offset = def_async_trip_offset;
+
+		polling_trip -= poll_offset;
+
+		for (unsigned int i = 0; i < sensors.size(); ++i) {
+			cthd_sensor *sensor;
+			sensor = sensors[i];
+
+			sensor->set_threshold(0, polling_trip);
+			// If the poll trip is already present then simply update
+			// the trip, instead of creating a new one.
+			cthd_trip_point trip_pt_polling(trip_points.size(), POLLING,
+					polling_trip, 0, index, sensor->get_index());
+			trip_pt_polling.thd_trip_point_set_control_type(PARALLEL);
+			trip_points.push_back(trip_pt_polling);
+		}
+	}
+}
+
 int cthd_zone::zone_update() {
 	int ret;
 
-	thd_model.set_zone_type(type_str);
-	thd_model.use_pid();
-
 	if (zone_bind_sensors() != THD_SUCCESS) {
-		thd_log_warn("Zone update failed: unable to bind \n");
+		thd_log_warn("Zone update failed: unable to bind\n");
 		return THD_ERROR;
 	}
 	ret = read_trip_points();
@@ -144,8 +161,8 @@ int cthd_zone::zone_update() {
 	int usr_psv_temp = read_user_set_psv_temp();
 	if (usr_psv_temp > 0) {
 		cthd_trip_point trip_pt_passive(0, PASSIVE, usr_psv_temp, 0, index,
-				DEFAULT_SENSOR_ID);
-		update_trip_temp(trip_pt_passive);
+		DEFAULT_SENSOR_ID);
+		update_highest_trip_temp(trip_pt_passive);
 	}
 
 	ret = read_cdev_trip_points();
@@ -154,77 +171,7 @@ int cthd_zone::zone_update() {
 		// Don't bail out as they may be attached by thermal relation tables
 	}
 
-	if (trip_points.size()) {
-		unsigned int polling_trip = 0;
-		unsigned int max_trip_temp = 0;
-
-		std::sort(trip_points.begin(), trip_points.end(), trip_sort);
-		thd_log_info("Sorted trip dump :\n");
-		for (unsigned int i = 0; i < trip_points.size(); ++i) {
-			trip_points[i].trip_dump();
-		}
-
-		// Set the lowest trip point as the threshold for sensor async mode
-		// Use that the lowest point, after that we poll
-		if (trip_points.size())
-			polling_trip = trip_points[0].get_trip_temp();
-		for (unsigned int i = 0; i < trip_points.size(); ++i) {
-			if (polling_trip > trip_points[i].get_trip_temp())
-				polling_trip = trip_points[i].get_trip_temp();
-			if (trip_points[i].get_trip_type() == MAX)
-				max_trip_temp = trip_points[i].get_trip_temp();
-			thd_log_info("trip type: %d temp: %d \n",
-					trip_points[i].get_trip_type(),
-					trip_points[i].get_trip_temp());
-		}
-
-		if (polling_trip > def_async_trip_offset)
-			polling_trip -= def_async_trip_offset;
-
-		for (unsigned int i = 0; i < sensors.size(); ++i) {
-			cthd_sensor *sensor;
-			sensor = sensors[i];
-			if (sensor->check_async_capable()) {
-				if (max_trip_temp) {
-					unsigned int _polling_trip;
-					// We have to guarantee MAX, so we better
-					// wake up before, so that by the time
-					// we are notified, temp > max temp
-					thd_model.set_max_temperature(max_trip_temp);
-					_polling_trip = thd_model.get_hot_zone_trigger_point();
-					if (polling_trip) {
-						if (_polling_trip < polling_trip) {
-							if ((polling_trip - _polling_trip)
-									< def_async_trip_offset)
-								polling_trip = _polling_trip
-										- def_async_trip_offset;
-							else
-								polling_trip = _polling_trip;
-						}
-					} else
-						polling_trip = _polling_trip;
-				}
-
-				sensor->set_threshold(0, polling_trip);
-				cthd_trip_point trip_pt_polling(trip_points.size(), POLLING,
-						polling_trip, 0, index, sensor->get_index());
-				trip_pt_polling.thd_trip_point_set_control_type(PARALLEL);
-				trip_points.push_back(trip_pt_polling);
-			}
-		}
-	}
-
-	for (unsigned int i = 0; i < trip_points.size(); ++i) {
-		cthd_trip_point &trip_point = trip_points[i];
-		unsigned int set_point;
-		if (trip_point.get_trip_type() == MAX) {
-			thd_model.set_max_temperature(trip_point.get_trip_temp());
-			set_point = thd_model.get_set_point();
-			if (set_point != thd_model.get_set_point()) {
-				trip_point.thd_trip_update_set_point(set_point);
-			}
-		}
-	}
+	sort_and_update_poll_trip();
 
 	return THD_SUCCESS;
 }
@@ -253,21 +200,21 @@ void cthd_zone::zone_temperature_notification(int type, int data) {
 	read_zone_temp();
 }
 
-void cthd_zone::zone_reset() {
+void cthd_zone::zone_reset(int force) {
 	int i, count;
 
 	if (zone_active) {
 		count = trip_points.size();
 		for (i = count - 1; i >= 0; --i) {
 			cthd_trip_point &trip_point = trip_points[i];
-			trip_point.thd_trip_cdev_state_reset();
+			trip_point.thd_trip_cdev_state_reset(force);
 		}
 	}
 }
 
 int cthd_zone::bind_cooling_device(trip_point_type_t type,
 		unsigned int trip_temp, cthd_cdev *cdev, int influence,
-		int sampling_period) {
+		int sampling_period, int target_state_valid, int target_state) {
 	int i, count;
 	bool added = false;
 
@@ -278,8 +225,8 @@ int cthd_zone::bind_cooling_device(trip_point_type_t type,
 		if ((trip_point.get_trip_type() == type)
 				&& (trip_point.get_trip_temp() > 0)
 				&& (trip_temp == 0 || trip_point.get_trip_temp() == trip_temp)) {
-			trip_point.thd_trip_point_add_cdev(*cdev,
-					cthd_trip_point::default_influence, sampling_period);
+			trip_point.thd_trip_point_add_cdev(*cdev, influence,
+					sampling_period, target_state_valid, target_state);
 			added = true;
 			zone_cdev_set_binded();
 			break;
@@ -337,37 +284,52 @@ int cthd_zone::update_psv_temperature(int psv_temp) {
 	return THD_SUCCESS;
 }
 
-void cthd_zone::add_trip(cthd_trip_point &trip) {
+void cthd_zone::add_trip(cthd_trip_point &trip, int force) {
+	if (force) {
+		trip_points.push_back(trip);
+		sort_and_update_poll_trip();
+		return;
+	}
+
 	bool add = true;
 	for (unsigned int j = 0; j < trip_points.size(); ++j) {
 		if (trip_points[j].get_trip_type() == trip.get_trip_type()) {
-			thd_log_debug("updating existing trip temp \n");
+			thd_log_debug("updating existing trip temp\n");
 			trip_points[j] = trip;
-			if (trip.get_trip_type() == MAX) {
-				thd_model.set_max_temperature(trip.get_trip_temp());
-				// TODO: If sensor supports polling
-				// update the polling threshold also.
-			}
 			add = false;
 			break;
 		}
 	}
 	if (add)
 		trip_points.push_back(trip);
+
+	sort_and_update_poll_trip();
+}
+
+void cthd_zone::update_highest_trip_temp(cthd_trip_point &trip)
+{
+	if (trip_points.size()) {
+		thd_log_info("trip_points.size():%zu\n", trip_points.size());
+		for (unsigned int j = trip_points.size() - 1;; --j) {
+			if (trip_points[j].get_trip_type() == trip.get_trip_type()) {
+				thd_log_info("updating existing trip temp\n");
+				trip_points[j].update_trip_temp(trip.get_trip_temp());
+				trip_points[j].update_trip_hyst(trip.get_trip_hyst());
+				break;
+			}
+		}
+		sort_and_update_poll_trip();
+	}
 }
 
 void cthd_zone::update_trip_temp(cthd_trip_point &trip) {
 	for (unsigned int j = 0; j < trip_points.size(); ++j) {
 		if (trip_points[j].get_trip_type() == trip.get_trip_type()) {
-			thd_log_debug("updating existing trip temp \n");
+			thd_log_debug("updating existing trip temp\n");
 			trip_points[j].update_trip_temp(trip.get_trip_temp());
 			trip_points[j].update_trip_hyst(trip.get_trip_hyst());
-			if (trip.get_trip_type() == MAX) {
-				thd_model.set_max_temperature(trip.get_trip_temp());
-				// TODO: If sensor supports polling
-				// update the polling threshold also.
-			}
 			break;
 		}
 	}
+	sort_and_update_poll_trip();
 }
