@@ -44,25 +44,31 @@
 #undef LOG_TAG
 #define LOG_TAG "THERMALD"
 #include <utils/Log.h>
-#include <cutils/log.h>
+#include <log/log.h>
 #include <cutils/properties.h>
 
 #define thd_log_fatal	ALOGE
 #define thd_log_error	ALOGE
 #define thd_log_warn	ALOGW
-#define thd_log_info	ALOGD
-#define thd_log_debug 	ALOGV
+#define thd_log_msg	ALOGI
+#if LOG_DEBUG_INFO == 1
+#define thd_log_info	ALOGI
+#define thd_log_debug 	ALOGD
+#else
+#define thd_log_info(...)
+#define thd_log_debug(...)
+#endif
 
 #else
 
 #include "config.h"
 
+// Keeping the logging flag enabled for non-android cases
+#define LOG_DEBUG_INFO	1
+
 #define LOCKF_SUPPORT
-#ifdef GLIBC_SUPPORT
+#ifdef GLIB_SUPPORT
 #include <glib.h>
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#include <dbus/dbus-glib.h>
 #include <glib/gi18n.h>
 #include <gmodule.h>
 
@@ -70,6 +76,7 @@
 #define thd_log_fatal		g_error		// Print error and terminate
 #define thd_log_error		g_critical
 #define thd_log_warn		g_warning
+#define thd_log_msg		g_message
 #define thd_log_debug		g_debug
 #define thd_log_info(...)	g_log(NULL, G_LOG_LEVEL_INFO, __VA_ARGS__)
 #else
@@ -79,6 +86,7 @@ static int dummy_printf(const char *__restrict __format, ...) {
 #define thd_log_fatal		printf
 #define thd_log_error		printf
 #define thd_log_warn		printf
+#define thd_log_msg		printf
 #define thd_log_debug		dummy_printf
 #define thd_log_info		printf
 #endif
@@ -98,5 +106,9 @@ class cthd_engine;
 class cthd_engine_therm_sysfs;
 extern cthd_engine *thd_engine;
 extern int thd_poll_interval;
-
+extern bool thd_ignore_default_control;
+extern bool workaround_enabled;
+extern bool disable_active_power;
+extern bool ignore_critical;
+extern bool power_floor_enable;
 #endif
